
#ifndef DISTRIBUTORCORE_CRYPTOCOMMONCRYPTO_HPP
#define DISTRIBUTORCORE_CRYPTOCOMMONCRYPTO_HPP 1


#include "DistributorBroadcastCore.hpp"


#if HAVE_COMMONCRYPTO > 0
/**
\brief
*/
class DistributorCore_CryptoCommonCrypto :
public DistributorCore_Crypto {
private:

	class Impl;
	SHARED_PTR< Impl> pimpl;

public:

	DistributorCore_CryptoCommonCrypto( DistributorCore_Adapter& adapter);
	~DistributorCore_CryptoCommonCrypto( void);

	//! Nach RFC2898 klartext Passphrase zu AES128 Schlüssel verwurschdeln. Z.B. OpenSSL::PKCS5_PBKDF2_HMAC_SHA1()
	bool set_passphrase(
		const uint8_t* const passphrase,
		size_t passphrase_len,
		const uint8_t* const salt,
		size_t salt_len,
		int iter);
	//! hmac = SHA256( data + RFC2898( Passphrase)); Z.B. OpenSSL::HMAC() / OpenSSL::HMAC_Update()
	bool make_hmac(
		const struct iovec* const data,
		unsigned int data_cnt,
		uint8_t* hmac);
	//! hmac == SHA256( data + RFC2898( Passphrase)); Z.B. OpenSSL::HMAC() / OpenSSL::HMAC_Update()
	bool check_hmac(
		const uint8_t* const data,
		size_t len,
		const uint8_t* const hmac);
	//! AES128(). Z.B. OpenSSL::EVP_CipherUpdate()
	size_t encrypt(
		const struct iovec* const clear_text,
		unsigned int clear_text_cnt,
		const uint8_t* const init_vect,
		uint8_t* cipher_text,
		size_t& cipher_text_len);
	//! AES128(). Z.B. OpenSSL::EVP_CipherUpdate()
	bool decrypt(
		const uint8_t* const cipher_text,
		const uint8_t* const init_vect,
		uint8_t* const clear_text,
		size_t& len);

};
#endif // #if HAVE_COMMONCRYPTO > 0


#endif // DISTRIBUTORCORE_CRYPTOCOMMONCRYPTO_HPP

